import sys

from status import UNKNOWN, STATUS, OK
from exceptions import StatusNotExist, ParameterInvalid
from perf import PerformanceData


class NagiosPlugin(object):
    """ Nagios plugin object
    """

    def __init__(self, name, print_header=True, print_perf=True):
        """
        :param name: name of the plugin
        :param print_header: True for print header
        :param print_perf: True for print performance data
        """
        object.__init__(self)
        self._perf_data = []
        self._status = UNKNOWN
        self._name = name
        self._print_header = print_header
        self._print_perf = print_perf

    @property
    def status(self):
        return self._status

    def start(self):
        """ start the plugin automatic:
        - check all performance data
        - print result string
        - exit with status code

        if something go wrong, print the exception string and exit with status
        UNKNOWN
        """
        try:
            self.main()
        except Exception as e:
            self.output(UNKNOWN, str(e))

    def main(self):
        """ main function, will be call by starting
        """
        self._status, msg_header, msg_perf = self.check_all()
        msg = "{} {}".format(self._name, STATUS[self.status].upper())
        if self._print_header and msg_header:
            msg += ": " + msg_header
        if self._print_perf and msg_perf:
            msg += " | " + msg_perf
        self.output(self.status, msg)

    def add(self, perf_data):
        """ add a performance data into the plugin

        :param perf_data: PerformanceData object
        """
        if not isinstance(perf_data, PerformanceData):
            raise ParameterInvalid("Input data isn't a PerformanceData object")
        self._perf_data.append(perf_data)

    def check_all(self):
        """ check all performance data and return the summary status, message
        header, performance data message

        :return: (status, msg_header, msg_perf)
        :rtype: tuple
        """
        if len(self._perf_data) < 1:
            raise Exception("There aren't any data to check")
        status = OK
        msg_header = ""
        msg_perf = ""
        for perf_data in self._perf_data:
            if msg_header:
                msg_header += ", "
                msg_perf += " "

            msg_header = perf_data.str_header()
            msg_perf = perf_data.str_perf()

            if perf_data.no_check:
                continue
            checked_status = perf_data.check()
            if checked_status > status:
                status = checked_status
        return status, msg_header, msg_perf

    @staticmethod
    def output(stt, msg):
        """ print message (msg) and exit with status code of stt

        :param stt: status id or status str
        :param msg: message to show
        """
        if isinstance(stt, int):
            status = stt
        elif isinstance(stt, str):
            if stt not in STATUS:
                raise StatusNotExist()
            status = STATUS[stt]
        else:
            raise StatusNotExist()
        sys.stdout.write("{}\n".format(msg))
        sys.exit(status)



from .status import STATUS


class ValueInvalid(Exception):
    pass


class ParameterInvalid(Exception):
    pass


class StatusNotExist(Exception):
    def __init__(self):
        super(StatusNotExist, self).__init__(
            "Only accept status " + str([stt for stt in STATUS]))



__author__ = 'viet.pham'
__copyright__ = 'GiQ GmbH'
__date__ = '30.07.2015'
__version__ = '0.0.1'
__status__ = 'beta'


from perf import PerformanceData
from plugin import NagiosPlugin
from status import OK, UNKNOWN, WARNING, CRITICAL
from exceptions import (ParameterInvalid, ValueInvalid, StatusNotExist)



from exceptions import ParameterInvalid, ValueInvalid
from status import OK, WARNING, CRITICAL


class PerformanceData(object):
    """ Nagios performance data
    """

    def __init__(self, name, value=None, warning=None, critical=None,
                 max_value='', min_value='', unit='', no_check=False,
                 type_check="asc"):
        """
        :param name: name of the performance data
        :param value: value
        :param warning: warning value
        :param critical: critical value
        :param max_value: max value to visualise
        :param min_value: min value to visualise
        :param unit: unit of the value
        :param no_check: True for ignore the result of this performance data
        :param type_check: 'asc' or 'desc', method to compare the value with
                           warning and critical values; default: 'asc'
        """
        object.__init__(self)
        if value is None:
            raise ParameterInvalid(
                "{} need a value".format(self.__class__.__name__))
        try:
            self.__value = int(value)
        except:
            raise ParameterInvalid("value invalid (only accept int or float)")
        self.type_check = type_check

        self.__name = name
        self.__warning = warning
        self.__critical = critical
        self.__max = max_value
        self.__min = min_value
        if unit != '' and unit not in ['s', 'us', 'ms', '%', 'B', 'KB', 'TB',
                                       'GB', 'c']:
            raise Exception("Unit invalid")
        self.__unit = unit
        self.e = None
        self.no_check = no_check

    @property
    def value(self):
        """ return value of the performance data (measuring)

        :return: value of the performance data (measuring)
        :rtype: float
        """
        return self.__value

    @value.setter
    def value(self, value):
        """ set the value of the performance data (measuring)

        :param value: value of the performance data (measuring)
        :type value: float
        """
        try:
            self.__value = float(value)
        except:
            raise ValueInvalid("value must be either int or float")

    def __repr__(self):
        """
        :return: performance data in string
        :rtype: str
        """
        return "{}={}{};{};{};{};{}".format(
            self.__name.strip().replace(' ', '_'), self.value,
            self.__unit if self.__unit is not None else '',
            self.__warning if self.__warning is not None else '',
            self.__critical if self.__critical is not None else '',
            self.__min if self.__min is not None else '',
            self.__max if self.__max is not None else '')

    def str_header(self):
        """ return the header of the result as string

        :return: header message
        :rtype: str
        """
        return "{} = {}{}".format(self.__name, self.value, self.__unit)

    def str_perf(self):
        """ return the performance data of the result as string

        :return: performance data
        :rtype: str
        """
        return self.__repr__()

    def check(self):
        """ check the performance data by comparing with warning and critical
        values
        :return: status code of this data/counter/gauge after checking
        :rtype: int
        """
        try:
            if self.type_check == "asc":
                if self.__critical is not None and self.__critical <= self.value:
                    return CRITICAL
                if self.__warning is not None and self.__warning <= self.value:
                    return WARNING
                return OK
            elif self.type_check == "desc":
                if self.__critical is not None and self.__critical >= self.value:
                    return CRITICAL
                if self.__warning is not None and self.__warning >= self.value:
                    return WARNING
                return OK
            else:
                return CRITICAL
        except:
            raise



__author__ = 'viet.pham'
__copyright__ = 'GiQ GmbH'
__date__ = '22.09.2015'
__version__ = '0.0.6'
__status__ = 'beta'

from distutils.core import setup

setup(
    name='nagios',
    version='0.0.7',
    packages=['nagios'],
    url='',
    author='Viet Pham',
    author_email='viet.pham@getgoods.de',
    description='module to develop plugin for nagios, icinga.',
    keywords=['nagios', 'icinga', 'plugin'],
)
